/****************************************************************************************
*  YAROCK                                                                               *
*  Copyright (c) 2010-2013 Sebastien amardeilh <sebastien.amardeilh+yarock@gmail.com>   *
*                                                                                       *
*  This program is free software; you can redistribute it and/or modify it under        *
*  the terms of the GNU General Public License as published by the Free Software        *
*  Foundation; either version 2 of the License, or (at your option) any later           *
*  version.                                                                             *
*                                                                                       *
*  This program is distributed in the hope that it will be useful, but WITHOUT ANY      *
*  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A      *
*  PARTICULAR PURPOSE. See the GNU General Public License for more details.             *
*                                                                                       *
*  You should have received a copy of the GNU General Public License along with         *
*  this program.  If not, see <http://www.gnu.org/licenses/>.                           *
*****************************************************************************************/

#ifndef _MPRIS2_H_
#define _MPRIS2_H_

#include <QDBusAbstractAdaptor>
#include <QObject>
#include <QtDBus>
#include <QString>
#include <QStringList>

#include "MediaItem/mediaitem.h"


class Mpris1;
/*
********************************************************************************
*                                                                              *
*    Mpris2                                                                    *
*                                                                              *
********************************************************************************
*/
/*class Mpris2Root;
class Mpris2Player;
class Mpris2TrackList;*/

class Mpris2 : public QObject
{
  Q_OBJECT

  public:
    Mpris2(Mpris1* mpris1, QObject* parent = 0);
    ~Mpris2();
};

/*
********************************************************************************
*                                                                              *
*    Mpris2Root                                                                *
*                                                                              *
********************************************************************************
*/
class Mpris2Root : public QDBusAbstractAdaptor
{
Q_OBJECT
Q_CLASSINFO("D-Bus Interface", "org.mpris.MediaPlayer2")

//org.mpris.MediaPlayer2 MPRIS 2.0 Root interface
Q_PROPERTY(bool CanQuit READ canQuit)
Q_PROPERTY(bool CanRaise READ canRaise)
Q_PROPERTY(QString DesktopEntry READ desktopEntry)
Q_PROPERTY(bool HasTrackList READ hasTrackList)
Q_PROPERTY(QString Identity READ identity)
Q_PROPERTY(QStringList SupportedMimeTypes READ supportedMimeTypes)
Q_PROPERTY(QStringList SupportedUriSchemes READ supportedUriSchemes)

public:
    Mpris2Root(QObject *parent = 0);
    virtual ~Mpris2Root();

    bool canQuit() const;
    bool canRaise() const;
    QString desktopEntry() const;
    bool hasTrackList() const;
    QString identity() const;
    QStringList supportedMimeTypes() const;
    QStringList supportedUriSchemes() const;

public slots:
     void Quit();
     void Raise();
};

/*
********************************************************************************
*                                                                              *
*    Mpris2Player                                                              *
*                                                                              *
********************************************************************************
*/
class Mpris2Player : public QDBusAbstractAdaptor
{
Q_OBJECT
Q_CLASSINFO("D-Bus Interface", "org.mpris.MediaPlayer2.Player")

//org.mpris.MediaPlayer2.Player MPRIS 2.0 Player interface
  Q_PROPERTY( QString PlaybackStatus READ PlaybackStatus )
  Q_PROPERTY( QString LoopStatus READ LoopStatus WRITE SetLoopStatus )
  Q_PROPERTY( double Rate READ Rate WRITE SetRate )
  Q_PROPERTY( bool Shuffle READ Shuffle WRITE SetShuffle )
  Q_PROPERTY( QVariantMap Metadata READ Metadata )
  Q_PROPERTY( double Volume READ Volume WRITE SetVolume )
  Q_PROPERTY( qlonglong Position READ Position )
  Q_PROPERTY( double MinimumRate READ MinimumRate )
  Q_PROPERTY( double MaximumRate READ MaximumRate )
  Q_PROPERTY( bool CanGoNext READ CanGoNext )
  Q_PROPERTY( bool CanGoPrevious READ CanGoPrevious )
  Q_PROPERTY( bool CanPlay READ CanPlay )
  Q_PROPERTY( bool CanPause READ CanPause )
  Q_PROPERTY( bool CanSeek READ CanSeek )
  Q_PROPERTY( bool CanControl READ CanControl )

public:
    Mpris2Player(Mpris1* mpris1, QObject *parent = 0);
    virtual ~Mpris2Player();

    // Player Properties
    QString PlaybackStatus() const;
    QString LoopStatus() const;
    void SetLoopStatus(const QString& value);
    double Rate() const;
    void SetRate(double value);
    bool Shuffle() const;
    void SetShuffle(bool value);
    QVariantMap Metadata() const;
    double Volume() const;
    void SetVolume(double value);
    qlonglong Position() const;
    double MaximumRate() const;
    double MinimumRate() const;
    bool CanGoNext() const;
    bool CanGoPrevious() const;
    bool CanPlay() const;
    bool CanPause() const;
    bool CanSeek() const;
    bool CanControl() const;

    // Methods
    void Next();
    void Previous();
    void Pause();
    void PlayPause();
    void Stop();
    void Play();
    void Seek(qlonglong offset);
    void SetPosition(const QDBusObjectPath& trackId, qlonglong offset);
    void OpenUri(const QString& uri);

private:
    QVariantMap GetMetadata(const MEDIA::TrackPtr track);
    void EmitNotification(const QString& name);

private slots:
  void slot_engineStateChanged();
  void slot_currentSongChanged();

private:
    QVariantMap m_last_metadata;
    Mpris1* m_mpris1;
};

/*
********************************************************************************
*                                                                              *
*    Mpris2TrackList                                                           *
*                                                                              *
********************************************************************************
*/
//! TODO


#endif // _MPRIS2_H_
